package EightBitAdder "A model of an 8-bit digital adder built from half adders and full adders."
  package Components "Components used in the 8-bit Adder example"
    extends Modelica.Icons.Package;

    model FullAdderCE "Full 1-bit Adder."
      import D = Modelica.Electrical.Digital;
      import L = Modelica.Electrical.Digital.Interfaces.Logic;
      Modelica.Electrical.Digital.Examples.Utilities.FullAdder adder(Adder1.delayTime = 0, Adder2.delayTime = 0.001, Adder1.XOR.y0 = Modelica.Electrical.Digital.Interfaces.Logic.'0', Adder1.AND.y0 = Modelica.Electrical.Digital.Interfaces.Logic.'0', Adder2.XOR.y0 = Modelica.Electrical.Digital.Interfaces.Logic.'0', Adder2.AND.y0 = Modelica.Electrical.Digital.Interfaces.Logic.'0') annotation(Placement(transformation(extent = {{0, -66.618}, {133.235, 66.618}}, rotation = 0, origin = {-53.235, -3.382}), visible = true));
      Modelica.Electrical.Digital.Interfaces.DigitalInput b annotation(Placement(visible = true, transformation(origin = {-96.873, 16.873}, extent = {{-6.873, -6.873}, {6.873, 6.873}}, rotation = 0), iconTransformation(origin = {50, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Digital.Interfaces.DigitalInput a annotation(Placement(visible = true, transformation(origin = {-96.771, 43.229}, extent = {{-6.771, -6.771}, {6.771, 6.771}}, rotation = 0), iconTransformation(origin = {-50, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Digital.Interfaces.DigitalInput c_in annotation(Placement(visible = true, transformation(origin = {-97.279, -50}, extent = {{-7.279, -7.279}, {7.279, 7.279}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Digital.Interfaces.DigitalOutput c_out annotation(Placement(visible = true, transformation(origin = {100, -50}, extent = {{-5.8, -5.8}, {5.8, 5.8}}, rotation = 0), iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
      Modelica.Electrical.Digital.Interfaces.DigitalOutput s annotation(Placement(visible = true, transformation(origin = {100.265, 43.28}, extent = {{-6.191, -6.191}, {6.191, 6.191}}, rotation = 0), iconTransformation(origin = {-0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = -450)));
    equation
      connect(adder.s, s) annotation(Line(visible = true, origin = {90.766, 43.265}, points = {{-10.1, -0.014}, {0.3, -0.015}, {0.3, 0.015}, {9.499, 0.015}}, color = {138, 45, 163}));
      connect(adder.c_out, c_out) annotation(Line(visible = true, origin = {90.596, -50.007}, points = {{-10.596, -0.008}, {0.596, -0.007}, {0.596, 0.007}, {9.404, 0.007}}, color = {138, 45, 163}));
      connect(c_in, adder.c_in) annotation(Line(visible = true, origin = {-65.746, -50.007}, points = {{-31.533, 0.007}, {9.511, 0.007}, {9.511, -0.007}, {12.511, -0.008}}, color = {138, 45, 163}));
      connect(b, adder.b) annotation(Line(visible = true, origin = {-65.645, 16.738}, points = {{-31.228, 0.135}, {9.41, 0.135}, {9.41, -0.135}, {12.41, -0.135}}, color = {138, 45, 163}));
      connect(a, adder.a) annotation(Line(visible = true, origin = {-65.619, 43.24}, points = {{-31.152, -0.011}, {9.384, -0.011}, {9.384, 0.011}, {12.384, 0.011}}, color = {138, 45, 163}));
      annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {212, 212, 212}, fillPattern = FillPattern.Solid, extent = {{-90, -90}, {90, 90}}), Text(visible = true, origin = {-71.031, 105}, extent = {{-11.031, -15}, {11.031, 15}}, textString = "a"), Text(visible = true, origin = {29.649, 103.121}, extent = {{-9.649, -13.121}, {9.649, 13.121}}, textString = "b"), Text(visible = true, origin = {106.606, 21.787}, extent = {{-13.394, -18.213}, {13.394, 18.213}}, textString = "cin"), Text(visible = true, origin = {1.433, 0}, extent = {{-68.567, -25.485}, {68.567, 25.485}}, textString = "FA"), Text(visible = true, origin = {-100, 23.121}, extent = {{-9.649, -13.121}, {9.649, 13.121}}, textString = "c"), Text(visible = true, origin = {19.649, -100}, extent = {{-9.649, -13.121}, {9.649, 13.121}}, textString = "s")}), Documentation(info = "<html><div>
<h3>Model Principles</h3>
</div>
<div>
<p>The full adder adds two 1-bit numbers together. In addition, the adder takes a 1-bit carry-in signal. Output from the binary addition is a 1-bit sum and a 1-bit carry-out.</p>
</div>
<div>
<p>The output from the adder is subject to the following truth table:</p>
</div>
<div>
<table style=\"border: 1px solid black; width: 50%;\">
<tbody>
<tr>
<th>a</th>
<th>b</th>
<th>c_in</th>
<th>c_out</th>
<th>s</th>
</tr>
<tr>
<th>1</th>
<th>0</th>
<th>0</th>
<th>0</th>
<th>1</th>
</tr>
<tr>
<th>0</th>
<th>1</th>
<th>0</th>
<th>0</th>
<th>1</th>
</tr>
<tr>
<th>0</th>
<th>0</th>
<th>1</th>
<th>0</th>
<th>1</th>
</tr>
<tr>
<th>1</th>
<th>1</th>
<th>0</th>
<th>1</th>
<th>0</th>
</tr>
<tr>
<th>0</th>
<th>1</th>
<th>1</th>
<th>1</th>
<th>0</th>
</tr>
<tr>
<th>1</th>
<th>0</th>
<th>1</th>
<th>1</th>
<th>0</th>
</tr>
<tr>
<th>1</th>
<th>1</th>
<th>1</th>
<th>1</th>
<th>1</th>
</tr>
<tr>
<th>0</th>
<th>0</th>
<th>0</th>
<th>0</th>
<th>0</th>
</tr>
</tbody>
</table>
</div></html>"));
    end FullAdderCE;

    model IntegerToBinaryConverter "Block that converts an integer number to a binary array."
      Modelica.Blocks.Interfaces.IntegerInput u annotation(Placement(visible = true, transformation(origin = {-40, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
      Modelica.Electrical.Digital.Interfaces.DigitalOutput y[n](start = fill(Modelica.Electrical.Digital.Interfaces.Logic.'0', n)) annotation(Placement(visible = true, transformation(origin = {40, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      parameter Integer n = 8 "Number of bits.";
    equation
      y = EightBitAdder.Components.IntegerToBinary(u, n);
      annotation(Documentation(info = "<html><div>
<h3>Model Principles</h3>
</div>
<div>
<p>This block takes an integer number and converts it into an array of size n consisting of 1-bit numbers. The parameter n, the total number of bits, can be specified so that the block can be used for conversions of arbitrary size. The maximum size of the integer that the block can take follows 2<sup>n</sup>-1 if the integer is unsigned or 2<sup>n-1</sup>-1 if the integer is signed.</p>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {194, 194, 194}, fillPattern = FillPattern.Solid, lineThickness = 2, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, origin = {-33.333, -33.333}, lineColor = {144, 0, 108}, fillColor = {144, 0, 108}, fillPattern = FillPattern.Solid, points = {{-66.667, 133.333}, {-66.667, -66.667}, {133.333, -66.667}})}), Diagram(coordinateSystem(extent = {{-40, -40}, {40, 40}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {-0.085, 25}, extent = {{-35.085, -26.956}, {35.085, 26.956}}, textString = "Textual Model")}));
    end IntegerToBinaryConverter;

    model BinaryToIntegerConverter "Block that converts a binary array to an integer number."
      import L = Modelica.Electrical.Digital.Interfaces.Logic;
      parameter Integer n = 8 "Number of bits.";
      Integer bit[n](start = fill(0, n)) "Internal state for bits.";
      Modelica.Electrical.Digital.Interfaces.DigitalInput x[n] annotation(Placement(visible = true, transformation(origin = {-40, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Interfaces.IntegerOutput y annotation(Placement(visible = true, transformation(origin = {40, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, -98.091}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    equation
      for i in 1:n loop
        if x[i] == Modelica.Electrical.Digital.Interfaces.Logic.'0' then
          bit[i] = 0;
        else
          bit[i] = 1;
        end if;
      end for;
      y = sum(bit[i] * integer(2 ^ (n - i)) for i in 1:n);
      annotation(Documentation(info = "<html><div>
<h3>Model Principles</h3>
</div>
<div>
<p>This block takes an array of size n consisting of 1-bit numbers and converts the array into an integer number. The parameter n, the total number of bits, can be specified so that the block can be used for conversions of arbitrary size. The maximum size of the integer that the block can take follows 2<sup>n</sup>-1 if the integer is unsigned or 2<sup>n-1</sup>-1 if the integer is signed.</p>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {194, 194, 194}, fillPattern = FillPattern.Solid, lineThickness = 2, extent = {{-100, -100}, {100, 100}}), Polygon(visible = true, origin = {-33.333, -33.333}, lineColor = {144, 0, 108}, fillColor = {144, 0, 108}, fillPattern = FillPattern.Solid, points = {{-66.667, 133.333}, {-66.667, -66.667}, {133.333, -66.667}})}), Diagram(coordinateSystem(extent = {{-40, -40}, {40, 40}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {0.085, 25}, extent = {{-35.085, -26.956}, {35.085, 26.956}}, textString = "Textual Model")}));
    end BinaryToIntegerConverter;

    function IntegerToBinary "Function that takes an integer number and converts it to an array of binary numbers."
      extends Modelica.Icons.Function;
      input Integer u;
      input Integer n;
      output Modelica.Electrical.Digital.Interfaces.Logic y[n] annotation(Placement(visible = true, transformation(origin = {147.902, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    protected
      Integer temp;
      Integer i;
    algorithm
      for j in 1:n loop
        y[j] := Modelica.Electrical.Digital.Interfaces.Logic.'0';
      end for;
      temp := u;
      i := -1;
      while temp >= 1 loop
        i := i + 1;
        if mod(temp, 2) == 0 then
          y[n - i] := Modelica.Electrical.Digital.Interfaces.Logic.'0';
        else
          y[n - i] := Modelica.Electrical.Digital.Interfaces.Logic.'1';
        end if;
        temp := integer(floor(temp / 2));
      end while;
      return;
      annotation(Documentation(info = "<html><div>
<h3>Function Documentation</h3>
</div>
<div>
<p>This algorithm takes an integer number and converts it into an array of binary numbers.</p>
</div>
<div>
<p>Used in the <a href=\"modelica://EightBitAdder.Components.IntegerToBinaryConverter#info\">IntegerToBinaryConverter</a> block.</p>
</div></html>"));
    end IntegerToBinary;

    model Adder "8-bit adder component."
      EightBitAdder.Components.FullAdderCE bit5 annotation(Placement(visible = true, transformation(origin = {-90, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.FullAdderCE bit4 annotation(Placement(visible = true, transformation(origin = {-30, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.FullAdderCE bit3 annotation(Placement(visible = true, transformation(origin = {29.86, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.FullAdderCE bit2 annotation(Placement(visible = true, transformation(origin = {89.86, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.FullAdderCE bit6 annotation(Placement(visible = true, transformation(origin = {-150, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -720)));
      EightBitAdder.Components.FullAdderCE bit1 annotation(Placement(visible = true, transformation(origin = {150, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.FullAdderCE bit0 annotation(Placement(visible = true, transformation(origin = {210, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -720)));
      EightBitAdder.Components.FullAdderCE bit7 annotation(Placement(visible = true, transformation(origin = {-210, -20}, extent = {{-20, -20}, {20, 20}}, rotation = -720)));
      EightBitAdder.Components.IntegerToBinaryConverter integer_a_to_binary annotation(Placement(visible = true, transformation(origin = {-100, 138.962}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      EightBitAdder.Components.IntegerToBinaryConverter integer_b_to_binary annotation(Placement(visible = true, transformation(origin = {80, 140.519}, extent = {{-20, -20}, {20, 20}}, rotation = -360)));
      Modelica.Electrical.Digital.Sources.Set set(x = Modelica.Electrical.Digital.Interfaces.Logic.'0') annotation(Placement(visible = true, transformation(origin = {290, -120}, extent = {{-20, -20}, {20, 20}}, rotation = -180)));
      EightBitAdder.Components.BinaryToIntegerConverter binaryToIntegerConverter annotation(Placement(visible = true, transformation(origin = {0, -127.192}, extent = {{-22.808, -22.808}, {22.808, 22.808}}, rotation = -360)));
      Modelica.Blocks.Interfaces.IntegerOutput y annotation(Placement(visible = true, transformation(origin = {223.84, -208.84}, extent = {{-23.84, -23.84}, {23.84, 23.84}}, rotation = -450), iconTransformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      Modelica.Blocks.Interfaces.IntegerInput b annotation(Placement(visible = true, transformation(origin = {80, 217.819}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {50, 110}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
      Modelica.Blocks.Interfaces.IntegerInput a annotation(Placement(visible = true, transformation(origin = {-100, 217.819}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {-50, 110}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
      Modelica.Electrical.Digital.Interfaces.DigitalOutput y1 annotation(Placement(visible = true, transformation(origin = {-303.864, -20}, extent = {{-6.136, -6.136}, {6.136, 6.136}}, rotation = -180), iconTransformation(origin = {-95.873, 0}, extent = {{-5.873, -5.873}, {5.873, 5.873}}, rotation = -180)));
      Modelica.Blocks.Math.IntegerToReal integerToReal annotation(Placement(visible = true, transformation(origin = {30, -180}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.RealToInteger realToInteger annotation(Placement(visible = true, transformation(origin = {120, -180}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Discrete.Sampler sampler(samplePeriod = 0.008, startTime = 0) annotation(Placement(visible = true, transformation(origin = {75, -180}, extent = {{-16.507, -16.507}, {16.507, 16.507}}, rotation = 0)));
    equation
      connect(integerToReal.y, sampler.u) annotation(Line(visible = true, origin = {48.096, -180}, points = {{-7.096, 0}, {7.096, 0}}, color = {1, 37, 163}));
      connect(sampler.y, realToInteger.u) annotation(Line(visible = true, origin = {100.579, -180}, points = {{-7.421, 0}, {7.421, 0}}, color = {1, 37, 163}));
      connect(realToInteger.y, y) annotation(Line(visible = true, origin = {197.29, -185.29}, points = {{-66.29, 5.29}, {0.87, 5.29}, {27.71, 5.29}, {26.55, -23.55}}, color = {255, 127, 0}));
      connect(binaryToIntegerConverter.y, integerToReal.u) annotation(Line(visible = true, origin = {6, -169.855}, points = {{-6, 20.29}, {-6, -10.145}, {12, -10.145}}, color = {255, 127, 0}));
      connect(bit7.c_out, y1) annotation(Line(visible = true, origin = {-266.932, -20}, points = {{36.932, 0}, {-36.932, 0}}, color = {138, 45, 163}));
      connect(bit7.s, binaryToIntegerConverter.x[1]) annotation(Line(visible = true, origin = {-105, -65.661}, points = {{-105, 25.661}, {-105, -14.339}, {105, -14.339}, {105, -38.723}}, color = {138, 45, 163}));
      connect(bit6.s, binaryToIntegerConverter.x[2]) annotation(Line(visible = true, origin = {-75, -65.661}, points = {{-75, 25.661}, {-75, -14.339}, {75, -14.339}, {75, -38.723}}, color = {138, 45, 163}));
      connect(bit5.s, binaryToIntegerConverter.x[3]) annotation(Line(visible = true, origin = {-45, -65.661}, points = {{-45, 25.661}, {-45, -14.339}, {45, -14.339}, {45, -38.723}}, color = {138, 45, 163}));
      connect(bit4.s, binaryToIntegerConverter.x[4]) annotation(Line(visible = true, origin = {-15, -65.661}, points = {{-15, 25.661}, {-15, -14.339}, {15, -14.339}, {15, -38.723}}, color = {138, 45, 163}));
      connect(bit3.s, binaryToIntegerConverter.x[5]) annotation(Line(visible = true, origin = {14.93, -65.661}, points = {{14.93, 25.661}, {14.93, -14.339}, {-14.93, -14.339}, {-14.93, -38.723}}, color = {138, 45, 163}));
      connect(bit2.s, binaryToIntegerConverter.x[6]) annotation(Line(visible = true, origin = {44.93, -65.661}, points = {{44.93, 25.661}, {44.93, -14.339}, {-44.93, -14.339}, {-44.93, -38.723}}, color = {138, 45, 163}));
      connect(bit1.s, binaryToIntegerConverter.x[7]) annotation(Line(visible = true, origin = {75, -65.661}, points = {{75, 25.661}, {75, -14.339}, {-75, -14.339}, {-75, -38.723}}, color = {138, 45, 163}));
      connect(bit0.s, binaryToIntegerConverter.x[8]) annotation(Line(visible = true, origin = {105, -65.661}, points = {{105, 25.661}, {105, -14.339}, {-105, -14.339}, {-105, -38.723}}, color = {138, 45, 163}));
      connect(bit6.c_out, bit7.c_in) annotation(Line(visible = true, origin = {-180, -20}, points = {{10, 0}, {-10, 0}}, color = {138, 45, 163}));
      connect(bit5.c_out, bit6.c_in) annotation(Line(visible = true, origin = {-120, -20}, points = {{10, 0}, {-10, 0}}, color = {138, 45, 163}));
      connect(bit4.c_out, bit5.c_in) annotation(Line(visible = true, origin = {-60, -20}, points = {{10, 0}, {-10, 0}}, color = {138, 45, 163}));
      connect(bit3.c_out, bit4.c_in) annotation(Line(visible = true, origin = {-0.07, -20}, points = {{9.93, 0}, {-9.93, 0}}, color = {138, 45, 163}));
      connect(bit2.c_out, bit3.c_in) annotation(Line(visible = true, origin = {59.86, -20}, points = {{10, 0}, {-10, 0}}, color = {138, 45, 163}));
      connect(bit1.c_out, bit2.c_in) annotation(Line(visible = true, origin = {119.93, -20}, points = {{10.07, 0}, {-10.07, 0}}, color = {138, 45, 163}));
      connect(bit0.c_out, bit1.c_in) annotation(Line(visible = true, origin = {180, -20}, points = {{10, 0}, {-10, 0}}, color = {138, 45, 163}));
      connect(set.y, bit0.c_in) annotation(Line(visible = true, origin = {237.071, -70}, points = {{32.929, -50}, {-1, -50}, {-1, 50}, {-7.071, 50}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[1], bit7.b) annotation(Line(visible = true, origin = {-60, 88.63}, points = {{140, 31.889}, {140, -28.63}, {-140, -28.63}, {-140, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[2], bit6.b) annotation(Line(visible = true, origin = {-30, 88.63}, points = {{110, 31.889}, {110, -28.63}, {-110, -28.63}, {-110, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[3], bit5.b) annotation(Line(visible = true, origin = {0, 88.63}, points = {{80, 31.889}, {80, -28.63}, {-80, -28.63}, {-80, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[4], bit4.b) annotation(Line(visible = true, origin = {30, 88.63}, points = {{50, 31.889}, {50, -28.63}, {-50, -28.63}, {-50, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[5], bit3.b) annotation(Line(visible = true, origin = {59.93, 88.63}, points = {{20.07, 31.889}, {20.07, -28.63}, {-20.07, -28.63}, {-20.07, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[6], bit2.b) annotation(Line(visible = true, origin = {89.93, 88.63}, points = {{-9.93, 31.889}, {-9.93, -28.63}, {9.93, -28.63}, {9.93, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[7], bit1.b) annotation(Line(visible = true, origin = {120, 88.63}, points = {{-40, 31.889}, {-40, -28.63}, {40, -28.63}, {40, -88.63}}, color = {138, 45, 163}));
      connect(integer_b_to_binary.y[8], bit0.b) annotation(Line(visible = true, origin = {150, 88.63}, points = {{-70, 31.889}, {-70, -28.63}, {70, -28.63}, {70, -88.63}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[1], bit7.a) annotation(Line(visible = true, origin = {-160, 68.24}, points = {{60, 50.722}, {60, -28.24}, {-60, -28.24}, {-60, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[2], bit6.a) annotation(Line(visible = true, origin = {-130, 68.24}, points = {{30, 50.722}, {30, -28.241}, {-30, -28.241}, {-30, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[3], bit5.a) annotation(Line(visible = true, origin = {-100, 96.481}, points = {{0, 22.481}, {0, -96.481}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[4], bit4.a) annotation(Line(visible = true, origin = {-70, 68.24}, points = {{-30, 50.722}, {-30, -28.24}, {30, -28.24}, {30, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[5], bit3.a) annotation(Line(visible = true, origin = {-40.07, 68.24}, points = {{-59.93, 50.722}, {-59.93, -28.241}, {59.93, -28.241}, {59.93, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[6], bit2.a) annotation(Line(visible = true, origin = {-10.07, 68.24}, points = {{-89.93, 50.722}, {-89.93, -28.241}, {89.93, -28.241}, {89.93, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[7], bit1.a) annotation(Line(visible = true, origin = {20, 68.24}, points = {{-120, 50.722}, {-120, -28.24}, {120, -28.24}, {120, -68.24}}, color = {138, 45, 163}));
      connect(integer_a_to_binary.y[8], bit0.a) annotation(Line(visible = true, origin = {50, 68.24}, points = {{-150, 50.722}, {-150, -28.241}, {150, -28.241}, {150, -68.24}}, color = {138, 45, 163}));
      connect(a, integer_a_to_binary.u) annotation(Line(visible = true, origin = {-100, 188.39}, points = {{0, 29.429}, {0, -29.428}}, color = {255, 127, 0}));
      connect(b, integer_b_to_binary.u) annotation(Line(visible = true, origin = {80, 189.169}, points = {{0, 28.65}, {0, -28.65}}, color = {255, 127, 0}));
      annotation(Documentation(info = "<html><div>
<h3>Model Principles</h3>
</div>
<div>
<p>The 8-bit adder model consists of eight <a href=\"modelica://EightBitAdder.Components.FullAdderCE#text\">1-bit adders</a> connected together in a series. The least significant bit of input a will be added together with the least significant bit of input b in the full adder located furthest to the right. Depending on the input, the adder will produce a 1-bit sum value that is the least significant bit of the result, as well as a 1-bit carry, which is sent to the next adder in the series. This is repeated in eight steps until 8 bits of output have been produced, as well as a possible carry-out signal.</p>
</div></html>"), __Wolfram(itemFlippingEnabled = true), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {212, 212, 212}, fillPattern = FillPattern.Solid, extent = {{-90, -90}, {90, 90}}), Text(visible = true, origin = {-94.703, 110}, extent = {{-24.703, -14.849}, {24.703, 14.849}}, textString = "Integer a"), Text(visible = true, origin = {10, 110}, extent = {{-24.703, -14.849}, {24.703, 14.849}}, textString = "Integer b"), Text(visible = true, origin = {0, -72.156}, extent = {{-24.703, -14.849}, {24.703, 14.849}}, textString = "Sum"), Text(visible = true, origin = {1.433, 34.515}, extent = {{-68.567, -25.485}, {68.567, 25.485}}, textString = "8 - BIT"), Text(visible = true, origin = {1.433, -20}, extent = {{-68.567, -25.485}, {68.567, 25.485}}, textString = "ADDER")}), Diagram(coordinateSystem(extent = {{-310, -210}, {310, 210}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}), graphics = {Text(visible = true, origin = {-155, 138.929}, extent = {{-32.5, -13.929}, {32.5, 13.929}}, textString = "integer_a_ToBinary", fontSize = 16), Text(visible = true, origin = {26.829, 141.071}, extent = {{-33.171, -13.929}, {33.171, 13.929}}, textString = "integer_b_ToBinary", fontSize = 16), Text(visible = true, origin = {-46, -151.071}, extent = {{-31, -13.929}, {31, 13.929}}, textString = "binaryToInteger", fontSize = 16), Text(visible = true, origin = {241.071, -70.837}, rotation = -90, extent = {{-20.837, -13.929}, {20.837, 13.929}}, textString = "carry in", fontSize = 16)}));
    end Adder;
    annotation(Documentation(info = "<html></html>"), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model EightBitAdderExample "Example setup of an 8-bit Adder component with integer input varying over time."
    extends Modelica.Icons.Example;
    EightBitAdder.Components.Adder eightBitAdder "Component that sums integers A and B together." annotation(Placement(visible = true, transformation(origin = {0, -20}, extent = {{-80, -80}, {80, 80}}, rotation = 0)));
    Integer calculatedSum;
    Modelica.Blocks.Sources.IntegerTable integer_table_a(table = transpose({0:size(tableA, 1) - 1, tableA})) "Value at position A to be summed." annotation(Placement(visible = true, transformation(origin = {-90, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.IntegerTable integer_table_b(table = transpose({0:size(tableA, 1) - 1, tableB})) "Value at position B to be summed." annotation(Placement(visible = true, transformation(origin = {-10, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Integer correctAnswer;
    parameter Integer tableA[:] = {21, 27, 31, 20, 17, 30, 22, 17, 29, 35} "Table of integers to add together with table B";
    parameter Integer tableB[:] = {50, 55, 49, 42, 56, 53, 59, 52, 47, 48} "Table of integers to add together with table A";
  equation
    connect(integer_table_b.y, eightBitAdder.b) annotation(Line(visible = true, origin = {27, 80}, points = {{-26, 10}, {13, 10}, {13, -20}}, color = {255, 127, 0}));
    connect(integer_table_a.y, eightBitAdder.a) annotation(Line(visible = true, origin = {-54.198, 80}, points = {{-24.802, 10}, {12.401, 10}, {12.401, -20}}, color = {255, 127, 0}));
    calculatedSum = eightBitAdder.y;
    correctAnswer = integer_table_a.y .+ integer_table_b.y;
    annotation(__Wolfram, Documentation(info = "<html>
<p>Binary adders are digital circuits often used in computers for basic arithmetic operations. Using the <a href=\"modelica://Modelica.Electrical.Digital#info\">digital package</a> in the standard Modelica library, an 8-bit adder has been constructed that takes two 8-bit integers and calculates their sum.</p>

<h4>Model Principles</h4>

<p>The 8-bit adder model consists of eight <a href=\"modelica://EightBitAdder.Components.FullAdderCE#info\">1-bit adders</a> connected together in a series. The least significant bit of input a will be added together with the least significant bit of input b in the full adder located furthest to the right. Depending on the input, the adder will produce a 1-bit sum value that is the least significant bit of the result, as well as a 1-bit carry, which is sent to the next adder in the series. This is repeated in eight steps until 8 bits of output have been produced, as well as a possible carry-out signal.</p>

<p>The input to the calculator is varied using <a href=\"modelica://Modelica.Blocks.Sources.IntegerTable#text\">IntegerTable</a> from the Modelica Standard Library. For each set of integers, the sum is calculated and simulated.&nbsp;</p>

<h4>Simulation</h4>

<p>To simulate the model, click the Simulate button in the top toolbar: <img src=\"modelica://EightBitAdder/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\" /></p>
<h3>Plot the Results</h3>

<p>When you run the simulation you will see the Sum and bit 0 Model plot:</p>

<p style=\"text-align: center;\"><img src=\"modelica://EightBitAdder/DocumentationFiles/adderplot.png\" style=\"width:600px\" alt=\"\" /></p>

<p>The plot displays the input and output from the adder converted to a real number, as well as the input states of the first bit.</p>
</html>", figures = {Figure(title = "Bit0", identifier = "8b0de", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit0.a, legend = "Bit 0 A"), Curve(y = eightBitAdder.bit0.b, legend = "Bit 0 B"), Curve(y = eightBitAdder.bit0.c_in, legend = "Bit 0 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit0.c_out, legend = "Bit 0 Carry Out"), Curve(y = eightBitAdder.bit0.s, legend = "Bit 0 Sum")}, x = Axis(min = 0, max = 10))}), Figure(title = "Bit1", identifier = "258f8", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit1.a, legend = "Bit 1 A"), Curve(y = eightBitAdder.bit1.b, legend = "Bit 1 B"), Curve(y = eightBitAdder.bit1.c_in, legend = "Bit 1 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit1.c_out, legend = "Bit 1 Carry Out"), Curve(y = eightBitAdder.bit1.s, legend = "Bit 1 Sum")}, x = Axis(min = 0, max = 10))}), Figure(title = "Sum and bit 0", identifier = "13265", preferred = true, plots = {Plot(curves = {Curve(y = eightBitAdder.a, legend = "Input A"), Curve(y = eightBitAdder.b, legend = "Input B"), Curve(y = eightBitAdder.y, legend = "Output")}), Plot(curves = {Curve(y = eightBitAdder.bit0.a, legend = "Bit 0 A"), Curve(y = eightBitAdder.bit0.b, legend = "Bit 0 B")})}), Figure(title = "Sum AB", identifier = "82b38", plots = {Plot(curves = {Curve(y = eightBitAdder.a, legend = "Input A"), Curve(y = eightBitAdder.b, legend = "Input B"), Curve(y = eightBitAdder.y, legend = "Output")})}), Figure(title = "Output and correct sum", identifier = "51a64", plots = {Plot(curves = {Curve(y = calculatedSum, legend = "Output"), Curve(y = correctAnswer, legend = "Correct Sum")})}), Figure(title = "Bit2", identifier = "287ea", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit2.a, legend = "Bit 2 A"), Curve(y = eightBitAdder.bit2.b, legend = "Bit 2 B"), Curve(y = eightBitAdder.bit2.c_in, legend = "Bit 2 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit2.c_out, legend = "Bit 2 Carry Out"), Curve(y = eightBitAdder.bit2.s, legend = "Bit 2 Sum")})}), Figure(title = "Bit3", identifier = "9971d", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit3.a, legend = "Bit 3 A"), Curve(y = eightBitAdder.bit3.b, legend = "Bit 3 B"), Curve(y = eightBitAdder.bit3.c_in, legend = "Bit 3 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit3.c_out, legend = "Bit 3 Carry Out"), Curve(y = eightBitAdder.bit3.s, legend = "Bit 3 Sum")})}), Figure(title = "Bit4", identifier = "5717b", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit4.a, legend = "Bit 4 A"), Curve(y = eightBitAdder.bit4.b, legend = "Bit 4 B"), Curve(y = eightBitAdder.bit4.c_in, legend = "Bit 4 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit4.c_out, legend = "Bit 4 Carry Out"), Curve(y = eightBitAdder.bit4.s, legend = "Bit 4 Carry In")})}), Figure(title = "Bit5", identifier = "7ca49", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit5.a, legend = "Bit 5 A"), Curve(y = eightBitAdder.bit5.b, legend = "Bit 5 B"), Curve(y = eightBitAdder.bit5.c_in, legend = "Bit 5 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit5.c_out, legend = "Bit 4 Carry Out"), Curve(y = eightBitAdder.bit5.s, legend = "Bit 4 Sum")})}), Figure(title = "Bit6", identifier = "13ca9", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit6.a, legend = "Bit 6 A"), Curve(y = eightBitAdder.bit6.b, legend = "Bit 6 B"), Curve(y = eightBitAdder.bit6.c_in, legend = "Bit 6 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit6.c_out, legend = "Bit 6 Carry Out"), Curve(y = eightBitAdder.bit6.s, legend = "Bit 6 Sum")}, x = Axis(min = 0, max = 10))}), Figure(title = "Bit7", identifier = "779a0", group = "Bit states", plots = {Plot(curves = {Curve(y = eightBitAdder.bit7.a, legend = "Bit 7 A"), Curve(y = eightBitAdder.bit7.b, legend = "Bit 7 B"), Curve(y = eightBitAdder.bit7.c_in, legend = "Bit 7 Carry In")}), Plot(curves = {Curve(y = eightBitAdder.bit7.c_out, legend = "Bit 7 Carry Out"), Curve(y = eightBitAdder.bit7.s, legend = "Bit 7 Sum")})})}), preferredView = "info", Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), experiment(StopTime = 10, __Wolfram_SynchronizeWithRealTime = false), Diagram(coordinateSystem(extent = {{-130, -105}, {115, 110}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end EightBitAdderExample;
  annotation(__Wolfram(openClass = EightBitAdder.EightBitAdderExample), preferredView = "info", Documentation(info = "<html></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, origin = {0, -1.5}, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 19.333}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), uses(Modelica(version = "4.0.0")), Diagram(coordinateSystem(extent = {{-148.5, -105}, {148.5, 105}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
end EightBitAdder;
